@@ -19,6 +19,10 @@
 #
 
 from __future__ import with_statement
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
+from __future__ import unicode_literals
 
 from contextlib import contextmanager
 
@@ -26,9 +30,21 @@ from . import ast
 
 
 class CCodeGenerator(object):
-    def __init__(self, namespace, out_h_filename, out_c_filename):
+    def __init__(self, namespace,
+                 out_h_filename,
+                 out_c_filename,
+                 function_decoration=[],
+                 include_first_header=[],
+                 include_last_header=[],
+                 include_first_src=[],
+                 include_last_src=[]):
         self.out_h_filename = out_h_filename
         self.out_c_filename = out_c_filename
+        self.function_decoration = function_decoration
+        self.include_first_header = include_first_header
+        self.include_last_header = include_last_header
+        self.include_first_src = include_first_src
+        self.include_last_src = include_last_src
         self._function_bodies = {}
         self.namespace = namespace
 
@@ -50,6 +66,10 @@ class CCodeGenerator(object):
         return param.type.ctype + suffix
 
     def _write_prelude(self, out, func):
+        if self.function_decoration:
+            out.write("""
+%s""" % " ".join(self.function_decoration))
+
         out.write("""
 %s
 %s (""" % (self._typecontainer_to_ctype(func.retval), func.symbol))
@@ -68,28 +88,31 @@ class CCodeGenerator(object):
         self._write_prelude(self.out_h, func)
         self.out_h.write(";\n\n")
 
-    def _write_annotation_transfer(self, transfer):
-        self.out_c.write("(transfer %s)" % (transfer, ))
+    def _write_annotation_transfer(self, node):
+        if (node.type not in ast.BASIC_TYPES or
+                node.type.ctype.endswith('*')):
+            self.out_c.write(" (transfer %s)" % (node.transfer, ))
 
     def _write_docs(self, func):
         self.out_c.write("/**\n * %s:\n" % (func.symbol, ))
         for param in func.parameters:
-            self.out_c.write(" * @%s: " % (param.argname, ))
+            self.out_c.write(" * @%s" % (param.argname, ))
             if param.direction in (ast.PARAM_DIRECTION_OUT,
                                    ast.PARAM_DIRECTION_INOUT):
                 if param.caller_allocates:
                     allocate_string = ' caller-allocates'
                 else:
                     allocate_string = ''
-                self.out_c.write("(%s%s) " % (param.direction,
-                                              allocate_string))
-                self._write_annotation_transfer(param.transfer)
+                self.out_c.write(": (%s%s) " % (param.direction,
+                                                allocate_string))
+                self._write_annotation_transfer(param)
             self.out_c.write(":\n")
         self.out_c.write(' *\n')
-        self.out_c.write(' * Undocumented.\n')
-        self.out_c.write(' *\n')
-        self.out_c.write(' * Returns: ')
-        self._write_annotation_transfer(func.retval.transfer)
+        self.out_c.write(' * Undocumented.')
+        if func.retval.type != ast.TYPE_NONE:
+            self.out_c.write('\n *\n')
+            self.out_c.write(' * Returns: ')
+            self._write_annotation_transfer(func.retval)
         self.out_c.write('\n */')
 
     @contextmanager
@@ -105,16 +128,31 @@ class CCodeGenerator(object):
         warning = '/* GENERATED BY testcodegen.py; DO NOT EDIT */\n\n'
         self.out_h.write(warning)
         nsupper = self.namespace.name.upper()
+
+        for header in self.include_first_header:
+            self.out_h.write("""#include "%s"\n""" % header)
+
         self.out_h.write("""
 #ifndef __%s_H__
 #define __%s_H__
 
 #include <glib-object.h>
+
 """ % (nsupper, nsupper))
 
+        for header in self.include_last_header:
+            self.out_h.write("""#include "%s"\n""" % header)
+
         self.out_c.write(warning)
+
+        for header in self.include_first_src:
+            self.out_c.write("""#include "%s"\n""" % header)
+
         self.out_c.write("""#include "%s"\n\n""" % (self.out_h_filename, ))
 
+        for header in self.include_last_src:
+            self.out_c.write("""#include "%s"\n""" % header)
+
     def _codegen_end(self):
         self.out_h.write("""#endif\n""")
 
@@ -131,7 +169,7 @@ class CCodeGenerator(object):
 
         self._codegen_start()
 
-        for node in self.namespace.itervalues():
+        for node in self.namespace.values():
             if isinstance(node, ast.Function):
                 with self._function(node):
                     body = self._function_bodies.get(node)
